@@ -3,15 +3,15 @@
 #' @keywords internal
 fastTickAgregation <- function (ts, alignBy = "minutes", alignPeriod = 1, tz = "GMT") {
   
-  if (alignBy == "secs" | alignBy == "seconds") {
+  if(alignBy == "ticks"){
+    return(na.locf(ts)[seqInclEnds(1, length(ts), alignPeriod)])
+  } else  if (alignBy == "secs" | alignBy == "seconds") {
     secs <- alignPeriod
     tby <- paste(alignPeriod, "sec", sep = " ")
-  } 
-  if (alignBy == "mins" | alignBy == "minutes") {
+  } else  if (alignBy == "mins" | alignBy == "minutes") {
     secs <- 60 * alignPeriod
     tby <- paste(60 * alignPeriod, "sec", sep = " ")
-  } 
-  if (alignBy == "hours"){
+  } else if (alignBy == "hours"){
     secs <- 3600 * alignPeriod
     tby <- paste(3600 * alignPeriod, "sec", sep = " ")
   }
@@ -36,7 +36,7 @@ fastTickAgregation <- function (ts, alignBy = "minutes", alignPeriod = 1, tz = "
 #' @importFrom data.table rbindlist setkey nafill set
 #' @keywords internal
 fastTickAgregation_DATA.TABLE <- function(dat, alignBy = "minutes", alignPeriod = 1, tz = "GMT"){
-  MAXDT <- .SD <- firstDT <- DT <- PRICE <- DATE <- NULL
+  .N <- MAXDT <- .SD <- firstDT <- DT <- DATE <- NULL
   if (alignBy == "secs" | alignBy == "seconds") {
     secs <- alignPeriod
   } 
@@ -59,6 +59,17 @@ fastTickAgregation_DATA.TABLE <- function(dat, alignBy = "minutes", alignPeriod
   ## These are used to make sure we dont have na's back and forth and to recude the chance we erroneously produce NA's
   dat <- dat[, lapply(.SD, nafill, type = "locf"), .SDcols = colnames(dat), by = list(DATE = as.Date(DT, tz = tz))]
   dat <- dat[, lapply(.SD, nafill, type = "nocb"), by = DATE]
+  
+  if(alignBy == "ticks"){ ## Special case for alignBy = "ticks"
+    if(alignPeriod == 1) return(dat[,!"DATE"])
+    if(alignPeriod < 1 | alignPeriod%%1 != 0){
+      stop("When alignBy is `ticks`, must be a positive integer valued numeric")
+    }
+    # if(length(unique(as.Date(pData[,DT]))) > 1){
+    #   stop("Multiday support for aggregatePrice with alignBy = \"ticks\" is not implemented yet.")
+    # }
+    return(dat[seqInclEnds(1, .N, alignPeriod), .SD, by = list(DATE = as.Date(DT)), .SDcols = 1:ncol(dat)][,!"DATE"])
+  }
   g <- dat[, list(DT = seq(first(DT), last(DT), by = secs, tz = tz), MAXDT = max(DT)), by = DATE]
   g$DT <- g$DT + (secs - as.numeric(g$DT, tz = tz) %% secs)
   out <- dat[g, roll = TRUE, on = "DT"][DT < MAXDT + secs]
@@ -94,44 +105,53 @@ fastTickAgregation_DATA.TABLE <- function(dat, alignBy = "minutes", alignPeriod
 #' @importFrom stats start end
 #' @keywords internal
 fastTickAgregation_RETURNS <- function (ts, alignBy = "minutes", alignPeriod = 1, tz = "GMT") {
-  
-  if (alignBy == "secs" | alignBy == "seconds") {
-    secs <- alignPeriod
-    tby <- paste(alignPeriod, "sec", sep = " ")
-  } 
-  if (alignBy == "mins" | alignBy == "minutes") {
-    secs <- 60 * alignPeriod
-    tby <- paste(60 * alignPeriod, "sec", sep = " ")
-  } 
-  if (alignBy == "hours"){
-    secs <- 3600 * alignPeriod
-    tby <- paste(3600 * alignPeriod, "sec", sep = " ")
-  }
-  g <- base::seq(start(ts), end(ts), by = tby)
-  rawg <- as.numeric(as.POSIXct(g, tz = tz))
-  newg <- rawg + (secs - rawg %% secs)
-  if(as.numeric(end(ts)) == newg[length(newg)-1]){
-    newg  <- newg[-length(newg)]
-  }
-  
-  firstObs <- ts[1,]
-  firstObs[] <- 0 ## We should have 0 return! We use [] to not just overwrite it as a numeric with 0
-  ts <- period.apply(ts, INDEX = merge.data.table(data.table(DT = index(ts), 1:nrow(ts)), data.table(DT = newg), by = "DT")$V2, 
-                     FUN = colSums, na.rm = TRUE)
-  ts <- na.locf(ts)
-  if(index(ts[1]) > index(firstObs)){
-    index(firstObs) <- index(ts[1]) - secs
-    ts <- c(firstObs, ts)
-  }
-  
-  return(ts)
+  DT <- NULL
+  agged <- fastTickAgregation_DATA.TABLE_RETURNS(as.data.table(ts)[, DT := index][, !"index"], alignBy = alignBy, alignPeriod = alignPeriod, tz = tz)
+  return(as.xts(agged))
+  # 
+  # 
+  # if (alignBy == "secs" | alignBy == "seconds") {
+  #   secs <- alignPeriod
+  #   tby <- paste(alignPeriod, "sec", sep = " ")
+  # } 
+  # if (alignBy == "mins" | alignBy == "minutes") {
+  #   secs <- 60 * alignPeriod
+  #   tby <- paste(60 * alignPeriod, "sec", sep = " ")
+  # } 
+  # if (alignBy == "hours"){
+  #   secs <- 3600 * alignPeriod
+  #   tby <- paste(3600 * alignPeriod, "sec", sep = " ")
+  # }
+  # if(alignBy != "ticks"){
+  #   g <- base::seq(start(ts), end(ts), by = tby)
+  #   rawg <- as.numeric(as.POSIXct(g, tz = tz))
+  #   newg <- rawg + (secs - rawg %% secs)
+  # } else {
+  #   newg <- index(ts)[seqInclEnds(1, NROW(ts), alignPeriod)]
+  # }
+  # if(as.numeric(end(ts)) == newg[length(newg)-1]){
+  #   newg  <- newg[-length(newg)]
+  # }
+  # 
+  # 
+  # firstObs <- ts[1,]
+  # firstObs[] <- 0 ## We should have 0 return! We use [] to not just overwrite it as a numeric with 0
+  # ts <- period.apply(ts, INDEX = merge.data.table(data.table(DT = index(ts), 1:nrow(ts)), data.table(DT = newg), by = "DT")$V2, 
+  #                    FUN = colSums, na.rm = TRUE)
+  # ts <- na.locf(ts)
+  # if(index(ts[1]) > index(firstObs)){
+  #   index(firstObs) <- index(ts[1]) - secs
+  #   ts <- c(firstObs, ts)
+  # }
+  # 
+  # return(ts)
 }
 
 
 #' @importFrom data.table rbindlist setkey nafill set
 #' @keywords internal
 fastTickAgregation_DATA.TABLE_RETURNS <- function(dat, alignBy = "minutes", alignPeriod = 1, tz = "GMT"){
-  DT_ROUND <- FIRST_DT <- .SD <- FIRST_DT <- DT <- DATE <- NULL
+  ....GRP.... <- .N <- DT_ROUND <- FIRST_DT <- .SD <- FIRST_DT <- DT <- DATE <- NULL
   if (alignBy == "secs" | alignBy == "seconds") {
     secs <- alignPeriod
   } 
@@ -154,8 +174,25 @@ fastTickAgregation_DATA.TABLE_RETURNS <- function(dat, alignBy = "minutes", alig
   ## These are used to make sure we dont have na's back and forth and to recude the chance we erroneously produce NA's
   dat <- dat[, lapply(.SD, nafill, type = "locf"), .SDcols = colnames(dat), by = list(DATE = as.Date(DT, tz = tz))]
   dat <- dat[, lapply(.SD, nafill, type = "nocb"), by = DATE]
+  if(alignBy == "ticks"){ ## Special case for alignBy = "ticks"
+    if(alignPeriod == 1) return(dat[,!"DATE"])
+    if(alignPeriod < 1 | alignPeriod%%1 != 0){
+      stop("When alignBy is `ticks`, must be a positive integer valued numeric")
+    }
+    dat[, ....GRP.... := tickGrouping_RETURNS(.N, alignPeriod), by = list(DATE = as.Date(DT))]
+    
+    expr <- setdiff(colnames(dat), c("DT","DATE", "....GRP...."))
+    expr <- paste0(sprintf("%s = sum(%s)", expr, expr), collapse = ", ")
+    expr <- sprintf("dat[, list(DT = DT[.N], %s), by = list(DATE = DATE, ....GRP....)]", expr)
+    dat <- eval(parse(text = expr))[, !c("DATE", "....GRP....")]
+    
+    return(dat[])
+  }
+  
   dat[, DT := as.numeric(DT, tz = tz)]
   dat[, FIRST_DT := min(DT), by = "DATE"]
+  
+  
   dat[, DT_ROUND := fifelse(DT == FIRST_DT,
                               floor(DT/secs) * secs,
                               ceiling(DT/secs) * secs)]
@@ -170,9 +207,6 @@ fastTickAgregation_DATA.TABLE_RETURNS <- function(dat, alignBy = "minutes", alig
 
 
 
-# # Necessary for check-package not throwing errors
-# #' @keywords internal
-# ..keepCols <- NULL
 # Necessary for check-package not throwing errors
 #' @keywords internal
 `:=` <- function(...) {
@@ -192,7 +226,7 @@ checkColumnNames <- function(data) {
     colnames(data) <-  toupper(colnames(data)) # Make sure we have upper case column names
     # Change column names to previous RTAQ format! 
     # Adjust Ask col naming:    
-    try((colnames(data)[xtsAttributes(data)[['Ask']]] = 'OFR'))
+    try((colnames(data)[xtsAttributes(data)[['ASK']]] = 'OFR'))
     # Adjust SYMBOL col naming:    
     try((colnames(data)[xtsAttributes(data)[['SYM_ROOT']]] = 'SYMBOL'))
     # Adjust Ask size col naming:
@@ -225,6 +259,7 @@ checkColumnNames <- function(data) {
     setnames(data, "ASKSIZE", "OFRSIZ", skip_absent = TRUE)
     # Adjust Bid size col naming:    
     setnames(data, "ASKSIZ", "OFRSIZ", skip_absent = TRUE)
+    setnames(data, "OFRSIZE", "OFRSIZ", skip_absent = TRUE)
     
     setnames(data, "TR_SCOND", "COND", skip_absent = TRUE)
     setnames(data, "CR", "CORR", skip_absent = TRUE)
@@ -246,14 +281,19 @@ checktData <- function(tData) {
   if (!any(colnames(tData) == "PRICE")) {
     stop("The argument tData should have a column containing the PRICE. Could not find that column.")
   }
-  if (!any(colnames(tData) == "SYMBOL")) {
-    warning("The argument tData should have a column containing SYMBOL. Could not find that column.")
+  if (!any(c("SYMBOL", "SYM_ROOT") %chin% colnames(tData))) {
+    warning("The argument tData should have a column containing SYMBOL (or SYM_ROOT). Could not find that column.")
   }
   
   if (is.data.table(tData)) {
     if (typeof(tData$PRICE) != "double") {
       stop("Column PRICE should be of type double.")
     }
+    if("SIZE" %chin% colnames(tData)){
+      if (typeof(tData$SIZE) != "integer" & typeof(tData$SIZE) != "double") {
+        stop("Column SIZE should be of type integer or double (after cleaning).")
+      }
+    }
   }
 }
 
@@ -267,11 +307,11 @@ checkqData <- function(qData) {
   if (!any(colnames(qData) == "BID")) {
     stop("The argument qData should have a column containing the BID. Could not find that column.")
   }
-  if (!any(colnames(qData) == "OFR")) {
+  if (!any(colnames(qData) %chin% c("OFR", "ASK"))) {
     stop("The argument qData should have a column containing the ASK / OFR. Could not find that column.")
   }
-  if (!any(colnames(qData) == "SYMBOL")) {
-    warning("The argument qData should have a column containing SYMBOL. Could not find that column.")
+  if (!any(c("SYMBOL", "SYM_ROOT") %chin% colnames(qData))) {
+    warning("The argument tData should have a column containing SYMBOL (or SYM_ROOT). Could not find that column.")
   }
   if (is.data.table(qData)) {
     if (typeof(qData$BID) != "double") {
@@ -290,7 +330,6 @@ checkqData <- function(qData) {
 #' @importFrom RcppRoll roll_median
 #' @keywords internal
 rollingMedianInclEnds <- function(x, weights, window, direction = "center") {
-  
   length_median_vec <- length(x)
   median_vec <- rep(NA, times = length_median_vec)
   halfwindow <- window / 2
@@ -408,8 +447,7 @@ BFMalgorithm <- function(tData, qData, backwardsWindow, forwardsWindow, plot, tz
   }
   
   tqData[c(whichBackwards, whichForwards), `:=`(DT = DT - c(na.omit(backwardsMatches), na.omit(forwardsMatches)))]
-  tqData[c(whichBackwards, whichForwards), `:=`(BID = NA)]
-  tqData[c(whichBackwards, whichForwards), `:=`(OFR = NA)]
+  tqData[c(whichBackwards, whichForwards), `:=`(BID = NA, OFR = NA)]
   
   
   
@@ -422,3 +460,239 @@ BFMalgorithm <- function(tData, qData, backwardsWindow, forwardsWindow, plot, tz
   return(list("tqData" = tqData, "forwardMatched" = cbind(forwardsMatches, whichForwards), 
               "backwardsMatched" = cbind(backwardsMatches, whichBackwards), "unmatched" = remaining))
 }
+
+
+seqInclEnds <- function(start, end, by){
+  val <- seq(1, end, by = by)
+  if(val[length(val)] != end){
+    val <- c(val, end)
+  }
+  return(val)
+}
+
+
+
+internalAggregateTSXTS <- function(ts, FUN = "previoustick", alignBy = "minutes", alignPeriod = 1, weights = NULL, dropna = FALSE, tz = NULL, ...){
+  makethispartbetter <- ((!is.null(weights))| alignBy=="days"| alignBy=="weeks" | (FUN!="previoustick") | dropna)
+  if(length(alignPeriod) > 1){
+    alignPeriod <- alignPeriod[1]
+    warning("alignPeriod must be of length one. Longer object provided. Using only first entry.")
+  }
+  
+  if (FUN == "previoustick") {
+    FUN <- previoustick 
+  } else {
+    FUN <- match.fun(FUN)
+  }
+  
+  if(is.null(tz)){
+    tz <- tzone(ts)
+  }
+  
+  if(alignBy == "ticks"){ ## Special case for alignBy = "ticks"
+    if(alignPeriod < 1 | alignPeriod%%1 != 0){
+      stop("When alignBy is `ticks`, must be a positive integer valued numeric")
+    }
+    idx <- seqInclEnds(1, NROW(ts), alignPeriod)
+    ts <- ts[idx,]
+    return(ts)
+  }
+  
+  if (makethispartbetter)  {
+    
+    if (is.null(weights)) {
+      ep <- endpoints(ts, alignBy, alignPeriod)
+      if (dim(ts)[2] == 1) { 
+        ts2 <- period.apply(ts, ep, FUN, ...) 
+      }
+      if (dim(ts)[2] > 1) {  
+        ts2 <- xts(apply(ts, 2, FUN = periodApply2, FUN2 = FUN, INDEX = ep, ...), order.by = index(ts)[ep],)
+      }
+    } else {
+      tsb <- cbind(ts, weights)
+      ep  <- endpoints(tsb, alignBy, alignPeriod)
+      ts2 <- period.apply(tsb, ep, FUN = match.fun(weightedaverage), ...)
+    }
+    if (alignBy == "minutes" | alignBy == "mins" | alignBy == "secs" | alignBy == "seconds") {
+      if (alignBy == "minutes" | alignBy == "mins") {
+        secs = alignPeriod * 60
+      }
+      if (alignBy == "secs" | alignBy == "seconds") {
+        secs <- alignPeriod
+      }
+      a <- index(ts2) + (secs - as.numeric(index(ts2)) %% secs)
+      ts3 <- xts(ts2, a, tzone = tz)
+    }
+    if (alignBy == "hours") {
+      secs = 3600
+      a <- index(ts2) + (secs - as.numeric(index(ts2)) %% secs)
+      ts3 <- xts(ts2, a, tzone = tz)
+    }
+    if (alignBy == "days") {
+      secs = 24 * 3600
+      a   <- index(ts2) + (secs - as.numeric(index(ts2)) %% secs) - (24 * 3600)
+      ts3 <- xts(ts2, a, tzone = tz)
+    }
+    if (alignBy == "weeks") {
+      secs = 24 * 3600 * 7
+      a <- (index(ts2) + (secs - (index(ts2) + (3L * 86400L)) %% secs)) - 
+        (24 * 3600)
+      ts3 <- xts(ts2, a, tzone = tz)
+    }
+    
+    if (!dropna) {
+      if (alignBy != "weeks" & alignBy != "days") {
+        if (alignBy == "secs" | alignBy == "seconds") {
+          tby <- "s"
+        }
+        if (alignBy == "mins" | alignBy == "minutes") {
+          tby <- "min"
+        }
+        if (alignBy == "hours") {
+          tby <- "h"
+        }
+        by <- paste(alignPeriod, tby, sep = " ")
+        allindex <- as.POSIXct(seq(start(ts3), end(ts3), by = by))
+        xx <- xts(rep("1", length(allindex)), order.by = allindex)
+        ts3 <- merge(ts3, xx)[, (1:dim(ts)[2])]
+      }
+    }
+    
+    ts3 <- xts(ts3, as.POSIXct(index(ts3)), tzone = tz)
+    return(ts3)
+  }
+  
+  if(!makethispartbetter){
+    if (alignBy == "secs" | alignBy == "seconds") { 
+      secs <- alignPeriod 
+      tby <- paste(alignPeriod, "sec", sep = " ")
+    }
+    if (alignBy == "mins" | alignBy == "minutes") { 
+      secs <- 60*alignPeriod
+      tby = paste(60*alignPeriod,"sec",sep=" ")
+    }
+    if (alignBy == "hours") { 
+      secs <- 3600 * alignPeriod 
+      tby <- paste(3600 * alignPeriod, "sec", sep=" ")
+    }
+    
+    g <- base::seq(start(ts), end(ts), by = tby)
+    rawg <- as.numeric(as.POSIXct(g, tz = "GMT"))
+    newg <- rawg + (secs - rawg %% secs)
+    
+    if(as.numeric(end(ts)) == newg[length(newg)-1]){
+      newg  <- newg[-length(newg)]
+    }
+    
+    
+    g <- as.POSIXct(newg, origin = "1970-01-01", tz = "GMT")
+    firstObs <- ts[1,]
+    ts <- na.locf(merge(ts, zoo(NULL, g)))[as.POSIXct(g, tz = tz)]
+    if(index(ts[1]) > index(firstObs)){
+      index(firstObs) <- index(ts[1]) - secs
+      ts <- c(firstObs, ts)
+    }
+    
+    ts <- ts[!duplicated(index(ts), fromLast = TRUE)]
+    
+    return(ts) 
+    }
+}
+
+
+internalAggregateTSDT <- function(ts, FUN = "previoustick", alignBy = "minutes", alignPeriod = 1, weights = NULL, dropna = FALSE, tz = NULL, ...){
+  .SD <- .N <- .I <- N <- DATE <- DT <- FIRST_DT <- DT_ROUND <- LAST_DT <- SYMBOL <- PRICE <- NULL
+  if (alignBy == "milliseconds") {
+    alignBy <- "secs"
+    alignPeriod <- alignPeriod / 1000
+  }
+  if(alignBy == "secs" | alignBy == "seconds"){
+    scaleFactor <- alignPeriod
+  }
+  if(alignBy == "mins" | alignBy == "minutes"){
+    scaleFactor <- alignPeriod * 60
+  }
+  if(alignBy == "hours"){
+    scaleFactor <- alignPeriod * 60 * 60
+  }
+  
+  if(! (alignBy %in% c("milliseconds", "secs", "seconds", "mins", "minutes", "hours", "ticks"))){
+    stop("alignBy not valid value. Valid values are: \"milliseconds\", \"secs\", \"seconds\", \"mins\", \"minutes\", \"hours\", and \"ticks\".")
+  }
+  
+  if (!("DT" %in% colnames(ts))) {
+    stop("Data.table neeeds DT column (date-time).")
+  }
+
+  if(alignBy == "ticks"){ ## Special case for alignBy = "ticks"
+    if(alignPeriod == 1) return(ts[])
+    
+    if(FUN != "previoustick") stop("Only previoustick function is supported when alignBy is `ticks`")
+    
+    if(alignPeriod < 1 | alignPeriod%%1 != 0){
+      stop("When alignBy is `ticks`, must be a positive integer valued numeric")
+    }
+    # if(length(unique(as.Date(pData[,DT]))) > 1){
+    #   stop("Multiday support for aggregatePrice with alignBy = \"ticks\" is not implemented yet.")
+    # }
+    return(ts[seqInclEnds(1, .N, alignPeriod), .SD, by = list(DATE = as.Date(DT)), .SDcols = 1:ncol(ts)][])
+  }
+  
+  timeZone <- format(ts$DT[1], format = "%Z")
+  if(is.null(timeZone) || timeZone == ""){
+    if(is.null(tz)){
+      tz <- "UTC"
+    }
+    if(!("POSIXct" %in% class(ts$DT))){
+      ts[, DT := as.POSIXct(format(DT, digits = 20, nsmall = 20), tz = tz)]
+    }
+  } else {
+    tz <- timeZone
+  }
+  setkeyv(ts, c("DT")) # The below code MAY fail with data with unordered DT column. Also setkey inceases speed of grouping
+  ## Checking ends
+  # Convert DT to numeric. This is much faster than dealing with the strings (POSIXct)
+  ts[, DT := as.numeric(DT, tz = tz)]
+  # Calculate the date in the data
+  ts[, DATE := as.Date(floor(DT / 86400), origin = "1970-01-01", tz = tz)]
+  
+  # Find the first observation per day.
+  ts[, FIRST_DT := min(DT), by = list(DATE)]
+  # Use Dirks answer here: https://stackoverflow.com/a/42498175 to round the timestamps to the latest scaleFactor
+  ts[, DT_ROUND := fifelse(DT == FIRST_DT,
+                              floor(DT/scaleFactor) * scaleFactor,
+                              ceiling(DT/scaleFactor) * scaleFactor)]
+  
+  ts[, LAST_DT := max(DT), by = list(DT_ROUND)]
+  
+  if(FUN == "previoustick"){
+    startsAndEnds <- ts[, list(START = first(DT_ROUND), END = last(DT_ROUND)), by = DATE]
+    dt_full_index <- data.table(DT_ROUND = as.numeric(mSeq(startsAndEnds[["START"]], startsAndEnds[["END"]], as.double(scaleFactor))))
+    
+    setkey(dt_full_index, "DT_ROUND")
+    
+    setkey(ts, DT)
+    
+    ts <- unique(ts[dt_full_index, roll = TRUE, on = "DT_ROUND"])
+    
+  }
+  
+  FUN <- match.fun(FUN)
+  if(!is.null(weights)){
+    if(NROW(weights) != NROW(ts) & NROW(weights) != 1){
+      stop("number of observbations in weights must match number of observations in ts")
+    }
+    for (col in setdiff(colnames(ts), c("DT", "DT_ROUND", "FIRST_DT", "LAST_DT", "DATE"))) {
+      set(ts, j = col, value = ts[[col]] * weights)
+    }
+  }
+  ts <- ts[, lapply(.SD, FUN, ...), by = list(DT_ROUND), .SDcols = setdiff(colnames(ts), c("DT", "DT_ROUND", "FIRST_DT", "LAST_DT", "DATE"))]
+  
+  
+  setnames(ts, "DT_ROUND", "DT")
+  set(ts, j = "DT", value = as.POSIXct(ts$DT, tz = tz, origin = "1970-01-01"))
+  
+  
+  return(ts[])
+  
+}
\ No newline at end of file
