@@ -28,8 +28,8 @@ test_that("quotesCleanup", {
 context("aggregatePrice")
 test_that("aggregatePrice", {
   expect_equal(
-    formatC(sum(head(aggregatePrice(sampleTData[, ], alignBy = "secs", alignPeriod = 30))$PRICE), digits = 10),
-    "     950.73"
+    formatC(sum(head(aggregatePrice(sampleTData, alignBy = "secs", alignPeriod = 30))$PRICE), digits = 10),
+    "     950.79"
   )
 })
 
@@ -47,38 +47,38 @@ test_that("selectExchange and data cleaning functions", {
   
   expect_equal(
     dim(rmTradeOutliersUsingQuotes(selectExchange(sampleTDataRaw, "P"), selectExchange(sampleQDataRaw, "N"), lagQuotes = 2)),
-    c(5502, 22)
+    c(5502, 12)
   )
   
   expect_equal(
     dim(rmLargeSpread(selectExchange(sampleQDataRaw, "N"))),
-    c(94422, 13)
+    c(94422, 7)
   )
   
   expect_equal(
     dim(mergeQuotesSameTimestamp(selectExchange(sampleQDataRaw, "N"), selection = "max.volume")),
-    c(46566, 13)
+    c(46566, 7)
   )
   
   expect_equal(
     dim(mergeQuotesSameTimestamp(selectExchange(sampleQDataRaw, "N"), selection = "weighted.average")),
-    c(46566, 13)
+    c(46566, 7)
   )
   
   expect_equal(
     dim(noZeroQuotes(selectExchange(sampleQDataRaw, "N"))),
-    c(94422, 13)
+    c(94422, 7)
   )
   
   
   expect_equal(
   dim(tradesCleanupUsingQuotes(tData = sampleTDataRaw, qData = sampleQData, lagQuotes = 2)),
-  c(72035, 23)
+  c(72035, 13)
   )
   
   expect_equal(
   dim(tradesCleanup(tDataRaw = sampleTDataRaw, exchanges = "N", report = FALSE)),
-  c(6140, 12)
+  c(7168, 8)
   )
 })
 
@@ -91,7 +91,7 @@ test_that("tradesCleanup gives same data as the shipped data", {
       tData = tradesCleanup(tDataRaw = sampleTDataRaw, exchanges = "N", report = FALSE),
       qData = quotesCleanup(qDataRaw = sampleQDataRaw, exchanges = "N", type = "standard", report = FALSE),
       lagQuotes = 0
-    )[, c("DT", "SYMBOL", "PRICE", "SIZE")]
+    )[, c("DT", "EX", "SYMBOL", "PRICE", "SIZE")]
   
   setkey(cleaned, SYMBOL, DT)
   expect_equal(cleaned, sampleTData)
@@ -139,7 +139,7 @@ test_that("tradesCleanup on-disk functionality", {
       tData = tradesCleanup(tDataRaw = sampleTDataRaw[as.Date(DT) == "2018-01-02"], exchanges = "N", report = FALSE),
       qData = quotesCleanup(qDataRaw = sampleQDataRaw[as.Date(DT) == "2018-01-02"], exchanges = "N", type = "standard", report = FALSE),
       lagQuotes = 0
-    )[, c("DT", "SYMBOL", "PRICE", "SIZE")]
+    )[, c("DT", "EX", "SYMBOL", "PRICE", "SIZE")]
   
   
   sampleTDataDay2 <-
@@ -147,12 +147,12 @@ test_that("tradesCleanup on-disk functionality", {
       tData = tradesCleanup(tDataRaw = sampleTDataRaw[as.Date(DT) == "2018-01-03"], exchanges = "N", report = FALSE),
       qData = quotesCleanup(qDataRaw = sampleQDataRaw[as.Date(DT) == "2018-01-03"], exchanges = "N", type = "standard", report = FALSE),
       lagQuotes = 0
-    )[, c("DT", "SYMBOL", "PRICE", "SIZE")]
+    )[, c("DT", "EX","SYMBOL", "PRICE", "SIZE")]
   
   
   
-  onDiskDay1 <- onDiskDay1[as.Date(DT, tz = "EST") == "2018-01-02",c("DT", "SYMBOL", "PRICE", "SIZE")][, DT := DT - 18000]
-  onDiskDay2 <- onDiskDay2[as.Date(DT, tz = "EST") == "2018-01-03",c("DT", "SYMBOL", "PRICE", "SIZE")][, DT := DT - 18000]
+  onDiskDay1 <- onDiskDay1[as.Date(DT, tz = "EST") == "2018-01-02",c("DT", "EX","SYMBOL", "PRICE", "SIZE")][, DT := DT - 18000]
+  onDiskDay2 <- onDiskDay2[as.Date(DT, tz = "EST") == "2018-01-03",c("DT", "EX","SYMBOL", "PRICE", "SIZE")][, DT := DT - 18000]
   setkey(onDiskDay1, SYMBOL, DT)
   setkey(onDiskDay2, SYMBOL, DT)
   expect_equal(onDiskDay1[,-"DT"], sampleTDataDay1[,-"DT"])
@@ -183,8 +183,13 @@ test_that("tradesCleanup on-disk functionality", {
 #   lines(old, col = "red", lwd = 1)
 # })
 
-
-
+context("aggregateTS xts vs data.table for previoustick")
+test_that("aggregateTS xts vs data.table for previoustick", {
+  res_DT <- aggregateTS(sampleTData[, list(DT, PRICE)])
+  expect_true(all.equal(as.xts(res_DT) , c(aggregateTS(as.xts(sampleTData[as.Date(DT) == "2018-01-02", list(DT, PRICE)])),
+                                         aggregateTS(as.xts(sampleTData[as.Date(DT) == "2018-01-03", list(DT, PRICE)]))))
+  )
+})
 
 context("aggregateTS edge cases")
 test_that("aggregateTS edge cases", {
@@ -202,6 +207,29 @@ test_that("aggregateTS edge cases", {
 })
 
 
+
+test_that("aggregateTS tick returns behave correctly",{
+  
+  ## Test using every second tick as well as two prime numbers since it's cheap and the primes should give the best coverage.
+  DT <- aggregateTS(sampleTData[, list(DT, PRICE)], alignBy = "ticks", alignPeriod = 2)
+  XTS <- aggregateTS(as.xts(sampleTData[, list(DT, PRICE)]), alignBy = "ticks", alignPeriod = 2)
+  expect_equal(as.numeric(DT$PRICE), as.numeric(XTS))
+  expect_true(all.equal(NROW(DT), NROW(XTS)))
+  
+  DT <- aggregateTS(sampleTData[, list(DT, PRICE)], alignBy = "ticks", alignPeriod = 3)
+  XTS <- aggregateTS(as.xts(sampleTData[, list(DT, PRICE)]), alignBy = "ticks", alignPeriod = 3)
+  expect_equal(as.numeric(DT$PRICE), as.numeric(XTS))
+  expect_true(all.equal(NROW(DT), NROW(XTS)))
+
+  DT <- aggregateTS(sampleTData[, list(DT, PRICE)], alignBy = "ticks", alignPeriod = 11)
+  XTS <- aggregateTS(as.xts(sampleTData[, list(DT, PRICE)]), alignBy = "ticks", alignPeriod = 11)
+  expect_equal(as.numeric(DT$PRICE), as.numeric(XTS))
+  expect_true(all.equal(NROW(DT), NROW(XTS)))
+  
+  
+})
+
+
 context("aggregatePrice time zones")
 test_that("aggregatePrice time zones", {
   dat <- data.table(DT = as.POSIXct(c(34150, 34201, 34201, 34500, 34500 + 1e-6, 34799, 34799, 34801, 34803, 35099), origin = "1970-01-01", tz = "EST"), PRICE = 0:9)
@@ -237,6 +265,16 @@ test_that("aggregatePrice edge cases", {
   expect_equal(output, target)
 })
 
+test_that("aggregatePrice filling different number of symbols over multiple days", {
+  foo <- rbind(sampleMultiTradeData, sampleMultiTradeData[SYMBOL != "ETF", list(DT = DT + 86400, SYMBOL, PRICE, SIZE)])
+  res <- aggregatePrice(foo, fill = TRUE, alignBy = "seconds")
+  expect_equal(nrow(res), 5 * 23401)
+  expect_equal(res[SYMBOL == "AAA" & as.Date(DT) == "2014-09-17"]$PRICE,
+               res[SYMBOL == "AAA" & as.Date(DT) == "2014-09-18"]$PRICE)
+  
+  
+})
+
 
 context("aggregatePrice milliseconds vs seconds")
 test_that("aggregatePrice milliseconds vs seconds", {
@@ -293,13 +331,13 @@ context("business time aggregation")
 test_that("business time aggregation",{
   skip_if_not(capabilities('long.double'), 'Skip tests when long double is not available')
   pData <- sampleTData
-  agged1 <- businessTimeAggregation(pData, measure = "intensity", obs = 390, bandwidth = 0.075)
-  expect_equal(nrow(agged1$pData), 780) # We return the correct number of observations
+  agged1 <- suppressWarnings(businessTimeAggregation(pData, measure = "intensity", obs = 390, bandwidth = 0.075))
+  expect_equal(nrow(agged1$pData), 779) # We return the correct number of observations
   
   
   expect_warning(businessTimeAggregation(pData, measure = "volume", obs = 390), "smaller")
   agged2 <- suppressWarnings(businessTimeAggregation(pData, measure = "volume", obs = 390))
-  expect_equal(nrow(agged2$pData), 748)
+  expect_equal(nrow(agged2$pData), 755)
   
   agged3 <- suppressWarnings(businessTimeAggregation(pData, measure = "vol", obs = 39, method = "PARM", RM = "rv", lookBackPeriod = 5))
   expect_equal(nrow(agged3$pData), 76)
@@ -326,13 +364,13 @@ test_that("refreshTime", {
   
   # Unit test for the refreshTime algorithm based on Kris' example in http://past.rinfinance.com/agenda/2015/workshop/KrisBoudt.pdf
   #suppose irregular timepoints: 
-  start = as.POSIXct("2010-01-01 09:30:00", tz = "GMT") 
-  ta = start + c(1, 2, 4, 5, 9, 14)
-  tb = start + c(1, 3, 6, 7, 8, 9, 10, 11, 15)
-  tc = start + c(1, 2, 3, 5, 7, 8, 10, 13)
-  a = as.xts(1:length(ta), order.by = ta) 
-  b = as.xts(1:length(tb), order.by = tb)
-  c = as.xts(1:length(tc), order.by = tc) 
+  start <- as.POSIXct("2010-01-01 09:30:00", tz = "GMT") 
+  ta <- start + c(1, 2, 4, 5, 9, 14)
+  tb <- start + c(1, 3, 6, 7, 8, 9, 10, 11, 15)
+  tc <- start + c(1, 2, 3, 5, 7, 8, 10, 13)
+  a <- as.xts(1:length(ta), order.by = ta)
+  b <- as.xts(1:length(tb), order.by = tb)
+  c <- as.xts(1:length(tc), order.by = tc)
   #Calculate the synchronized timeseries: 
   expected <- xts(matrix(c(1,1,1, 2,2,3, 4,3,4, 5,6,6, 6,8,8), ncol = 3, byrow = TRUE), order.by = start + c(1,3,6,9,14))
   colnames(expected) <- c("a", "b", "c")
@@ -361,36 +399,57 @@ test_that("refreshTime", {
   
   expect_equal(RT, expected)
   
+  
+  ## Make sure we take copies in the refreshTime() function when we make changes to colnames by reference.
+  ## If the copies are disabled in the code these tests fail.
+  expect_equal(colnames(aDT), c("DT", "PRICE"))
+  expect_equal(colnames(bDT), c("DT", "PRICE"))
+  expect_equal(colnames(cDT), c("DT", "PRICE"))
+  
   expect_equal(refreshTime(list("a" = aDT, "b" = bDT, "c" = cDT), sort = TRUE, criterion = "squared duration"), 
                expected[, names(expected)[c(1, sqDur + 1)], with = FALSE])
   expect_equal(refreshTime(list("a" = aDT, "b" = bDT, "c" = cDT), sort = TRUE, criterion = "duration"),
                expected[, names(expected)[c(1, dur + 1)], with = FALSE])
   
+  
+  
 })
 
 
 library(data.table)
 
-context("makeRMFormat")
-test_that("makeRMFormat",{
+context("spreadPrices and gatherPrices")
+test_that("spreadPrices",{
   set.seed(1)
   PRICE <- DT <- .N <- NULL
   data1 <- copy(sampleTData)[,  `:=`(PRICE = PRICE * runif(.N, min = 0.99, max = 1.01),
                                                  DT = DT + runif(.N, 0.01, 0.02))]
   data2 <- copy(sampleTData)[, SYMBOL := 'XYZ']
   
-  dat <- rbind(data1, data2)
-  setkey(dat, "DT")
-  dat <- makeRMFormat(dat)
+  dat1 <- rbind(data1, data2)
+  setkey(dat1, "DT")
+  dat <- spreadPrices(dat1)
   
   res <- rCov(dat, alignBy = 'minutes', alignPeriod = 5, makeReturns = TRUE, cor = TRUE)
-  target <- list("2018-01-02" = matrix(c(1, 0.05400510115,
-                                         0.05400510115, 1), ncol = 2),
-                 "2018-01-03" = matrix(c(1, 0.171321754,
-                                         0.171321754, 1), ncol = 2)
+  target <- list("2018-01-02" = matrix(c(1, 0.1936498028,
+                                         0.1936498028, 1), ncol = 2),
+                 "2018-01-03" = matrix(c(1, 0.1590385524,
+                                         0.1590385524, 1), ncol = 2)
                  )
   expect_equal(res, target)
+})
+test_that("gatherPrices and spreadPrices back and forth",{
+  set.seed(1)
+  PRICE <- DT <- .N <- NULL
+  data1 <- copy(sampleTData)[,  `:=`(PRICE = PRICE * runif(.N, min = 0.99, max = 1.01),
+                                     DT = DT + runif(.N, 0.01, 0.02))]
+  data2 <- copy(sampleTData)[, SYMBOL := 'XYZ']
   
+  dat1 <- rbind(data1, data2)[, list(DT, SYMBOL, PRICE)]
+  setkeyv(dat1, c("DT", "SYMBOL"))
+  dat <- spreadPrices(dat1)
+  expect_equal(dat1, gatherPrices(dat))
+  expect_equal(spreadPrices(gatherPrices(dat)), dat)
   
 })
 
@@ -418,6 +477,20 @@ test_that("Backwards-forwards matching algorithm produces correct result", {
                                   qData = quotesCleanup(qDataRaw = sampleQDataRaw, type = 'standard', report = FALSE, exchanges = 'N'), 
                                   BFM = TRUE, lagQuotes = 0)
   
-  expect_equal(dim(bfmMatched), c(19887, 18))
+  expect_equal(dim(bfmMatched), c(19888, 14))
 })
 
+
+
+context("seqInclEnds")
+test_that("seqInclEnds gives expected result", {
+  
+  target <- c(1,5,9,10)
+  expect_equal(target, seqInclEnds(1, 10, 4))
+  
+  target <- c(1, 4, 7, 10)
+  expect_equal(target, seqInclEnds(1, 10, 3))
+  
+  
+  
+})
